#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import sys

from . import base


class Loader(object):
    def __init__(self):
        self.migrations = []
        self.migrations_m = {}
        self.migrations_path = {}

    def __cmp__(self, value):
        return self.timestamp.__cmp__(value.timestamp)

    def __lt__(self, value):
        return self.timestamp < value.timestamp

    def __gt__(self, value):
        return self.timestamp > value.timestamp

    def __eq__(self, value):
        return self.timestamp == value.timestamp

    def __le__(self, value):
        return self.timestamp <= value.timestamp

    def __ge__(self, value):
        return self.timestamp >= value.timestamp

    def __ne__(self, value):
        return self.timestamp != value.timestamp

    def load(self):
        return self.migrations

    def upgrade(self, *args, **kwargs):
        migrations = self.load()

        db = base.Migratore.get_db(*args, **kwargs)
        try:
            timestamp = db.timestamp()
            timestamp = timestamp or 0
            for migration in migrations:
                is_valid = migration.timestamp > timestamp
                if not is_valid:
                    continue
                result = migration.start()
                if not result == "success":
                    break
        finally:
            db.close()

    def dry_upgrade(self, *args, **kwargs):
        migrations = self.load()

        db = base.Migratore.get_db(*args, **kwargs)
        try:
            timestamp = db.timestamp()
            timestamp = timestamp or 0
            for migration in migrations:
                is_valid = migration.timestamp > timestamp
                if not is_valid:
                    continue
                print(migration)
        finally:
            db.close()

    def rebuild(self, id, *args, **kwargs):
        self.load()
        migration = self.migrations_m[id]
        if not migration:
            raise RuntimeError("Migration '%s' not found" % id)
        migration.start(operation="run_partial")

    def touch(self, id, *args, **kwargs):
        from . import migration

        self.load()
        path = self.migrations_path.get(id)
        if not path:
            raise RuntimeError("Migration '%s' not found" % id)
        migration.Migration.touch_file(path)

    def skip(self, *args, **kwargs):
        migration = self.get_current_migration()
        migration.start(operation="run_skip")

    def get_current_migration(self):
        migrations = self.load()

        db = base.Migratore.get_db()
        try:
            timestamp = db.timestamp()
            timestamp = timestamp or 0
            for migration in migrations:
                if migration.timestamp > timestamp:
                    return migration
        finally:
            db.close()

        raise RuntimeError("No current migration found")

    def get_migration(self, timestamp):
        migrations = self.load()
        for migration in migrations:
            if migration.timestamp == timestamp:
                return migration
        raise RuntimeError("No migration found for timestamp %d" % timestamp)

    def get_migration_by_uuid(self, uuid):
        migrations = self.load()
        for migration in migrations:
            if migration.uuid == uuid:
                return migration
        raise RuntimeError("No migration found for UUID %s" % uuid)

    def get_migration_by_any(self, timestamp_or_uuid):
        migrations = self.load()
        for migration in migrations:
            if migration.timestamp == timestamp_or_uuid:
                return migration
            if migration.uuid == timestamp_or_uuid:
                return migration
        raise RuntimeError(
            "No migration found for identifier %s" % str(timestamp_or_uuid)
        )


class DirectoryLoader(Loader):
    def __init__(self, path):
        Loader.__init__(self)
        self.path = path

    def load(self):
        names = []
        modules = []

        sys.path.insert(0, self.path)

        files = os.listdir(self.path)

        for file in files:
            base, extension = os.path.splitext(file)
            if not extension == ".py":
                continue
            names.append((base, file))

        for name, file in names:
            module = __import__(name)
            modules.append((module, file))

        for module, file in modules:
            if not hasattr(module, "migration"):
                continue
            migration = getattr(module, "migration")
            self.migrations.append(migration)
            self.migrations_m[migration.uuid] = migration
            self.migrations_m[str(migration.timestamp)] = migration
            self.migrations_path[migration.uuid] = os.path.join(self.path, file)
            self.migrations_path[str(migration.timestamp)] = os.path.join(
                self.path, file
            )

        self.migrations.sort(key=lambda item: item.timestamp)
        return self.migrations



#!/usr/bin/python
# -*- coding: utf-8 -*-

from . import base
from . import cli
from . import info
from . import loader
from . import migration
from . import mysql

from .base import Migratore, Console, Database, Table, Result
from .cli import (
    run_help,
    run_version,
    run_environ,
    run_list,
    run_errors,
    run_trace,
    run_rebuild,
    run_upgrade,
    run_generate,
    main,
)
from .info import (
    NAME,
    VERSION,
    AUTHOR,
    EMAIL,
    DESCRIPTION,
    LICENSE,
    KEYWORDS,
    URL,
    COPYRIGHT,
)
from .loader import Loader, DirectoryLoader
from .migration import Migration
from .mysql import MySQLDatabase



#!/usr/bin/python
# -*- coding: utf-8 -*-

import migratore


class Migration(migratore.Migration):
    def __init__(self):
        migratore.Migration.__init__(self)
        self.uuid = "59a287c0-36a5-4dd7-898b-b421d3ea8d81"
        self.timestamp = 1391804600
        self.description = "initial setup and provision of the database"

    def run(self, db):
        migratore.Migration.run(self, db)

        self.begin("creating schema")
        table = db.create_table("users")
        table.add_column("username", type="text")
        table.add_column("password", type="text")
        self.end("creating schema")

        def task(table, index):
            object_id = index + 1
            username = "username-%d" % object_id
            password = "password-%d" % object_id
            table.insert(object_id=object_id, username=username, password=password)

        table.run(task, 1000, title="provisioning schema")


migration = Migration()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import sys
import setuptools

BASE_PATH = os.path.realpath(__file__)
BASE_DIR = os.path.dirname(BASE_PATH)
SRC_DIR = os.path.join(BASE_DIR, "src")
MIGRATORE_DIR = os.path.join(SRC_DIR, "migratore")
sys.path.insert(0, SRC_DIR)
sys.path.insert(0, MIGRATORE_DIR)

import info

setuptools.setup(
    name=info.NAME,
    version=info.VERSION,
    author=info.AUTHOR,
    author_email=info.EMAIL,
    description=info.DESCRIPTION,
    license=info.LICENSE,
    keywords=info.KEYWORDS,
    url=info.URL,
    zip_safe=True,
    packages=["migratore", "migratore.examples", "migratore.examples.migrations"],
    test_suite="migratore.test",
    package_dir={"": os.path.normpath("src")},
    package_data={"migratore": ["templates/*"]},
    entry_points={"console_scripts": ["migratore = migratore.cli:main"]},
    install_requires=["legacy"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Utilities",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.0",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    long_description=open(os.path.join(os.path.dirname(__file__), "README.md"), "rb")
    .read()
    .decode("utf-8"),
    long_description_content_type="text/markdown",
)
